@@ -1,5 +1,6 @@
 package Bitcoin::Crypto::DerivationPath;
-$Bitcoin::Crypto::DerivationPath::VERSION = '3.000';
+$Bitcoin::Crypto::DerivationPath::VERSION = '3.999_1'; # TRIAL
+$Bitcoin::Crypto::DerivationPath::VERSION = '3.9991';
 use v5.10;
 use strict;
 use warnings;
@@ -11,6 +12,8 @@ use Types::Common -sigs, -types;
 use Bitcoin::Crypto::Constants;
 use Bitcoin::Crypto::Exception;
 
+use namespace::clean;
+
 has param 'private' => (
 	isa => Bool,
 );
@@ -56,9 +59,9 @@ sub get_path_hardened
 	];
 }
 
-signature_for get_path_hardened => (
-	method => Object,
-	positional => [],
+signature_for from_string => (
+	method => Str,
+	positional => [Str],
 );
 
 sub from_string
