@@ -1,5 +1,6 @@
 package Bitcoin::Crypto::Key::Private;
-$Bitcoin::Crypto::Key::Private::VERSION = '3.000';
+$Bitcoin::Crypto::Key::Private::VERSION = '3.999_1'; # TRIAL
+$Bitcoin::Crypto::Key::Private::VERSION = '3.9991';
 use v5.10;
 use strict;
 use warnings;
@@ -18,7 +19,7 @@ use Bitcoin::Crypto::Exception;
 
 use namespace::clean;
 
-with qw(Bitcoin::Crypto::Role::BasicKey);
+extends qw(Bitcoin::Crypto::Key::Base);
 
 sub _is_private { 1 }
 
@@ -110,6 +111,7 @@ sub get_public_key
 		compressed => $self->compressed,
 		network => $self->network,
 		purpose => $self->purpose,
+		taproot_output => $self->taproot_output,
 	);
 
 	return $public;
@@ -142,7 +144,7 @@ Bitcoin::Crypto::Key::Private - Bitcoin private keys
 	# use to_format to get the representation you need
 
 	use Bitcoin::Crypto::Util qw(to_format);
-	my $sig_hex =  to_format [hex => $sig];
+	my $sig_hex = to_format [hex => $sig];
 
 	# signature verification
 
@@ -152,29 +154,51 @@ Bitcoin::Crypto::Key::Private - Bitcoin private keys
 
 This class allows you to create a private key instance.
 
-You can use a private key to:
+You can use a private key to get public keys and sign or verify transactions.
 
-=over
+=head1 INTERFACE
 
-=item * generate public keys
+=head2 Attributes
 
-=item * sign and verify messages
+=head3 compressed
 
-=back
+Boolean value indicating if this ECC key should be compressed. Default: C<true>.
 
-Please note that any keys generated are by default compressed.
+I<writer:> C<set_compressed>
 
-see L<Bitcoin::Crypto::Network> if you want to work with other networks than
-Bitcoin Mainnet.
+=head3 network
 
-=head1 METHODS
+Instance of L<Bitcoin::Crypto::Network> - current network for this key. Can be
+coerced from network id. Default: current default network.
 
-=head2 new
+I<writer:> C<set_network>
+
+=head3 purpose
+
+BIP44 purpose which was used to obtain this key. Filled automatically when
+deriving an extended key. If the key was not obtained through BIP44 derivation,
+this attribute is C<undef>.
+
+I<writer:> C<set_purpose>
+
+I<clearer:> C<clear_purpose>
+
+=head3 taproot_output
+
+Boolean value indicating if this key was obtained through taproot tweaking.
+Taproot output keys are used to sign and verify schnorr signatures in P2TR
+outputs. Default: C<false>
+
+I<writer:> C<set_taproot_output>
+
+=head2 Methods
+
+=head3 new
 
 Constructor is reserved for internal and advanced use only. Use L</from_serialized>
 or L</from_wif> instead.
 
-=head2 from_serialized
+=head3 from_serialized
 
 	$key_object = $class->from_serialized($serialized)
 
@@ -183,30 +207,14 @@ formatable bytestring containing the private key entropy.
 
 Returns a new key object instance.
 
-=head2 to_serialized
+=head3 to_serialized
 
 	$serialized = $key_object->to_serialized()
 
 This returns a private key as a sequence of bytes. The result is a bytestring
 which can be further formated with C<to_format> utility.
 
-=head2 from_bytes
-
-Deprecated. Use C<< $class->from_serialized($data) >> instead.
-
-=head2 to_bytes
-
-Deprecated. Use C<< $key->to_serialized() >> instead.
-
-=head2 from_hex
-
-Deprecated. Use C<< $class->from_serialized([hex => $data]) >> instead.
-
-=head2 to_hex
-
-Deprecated. Use C<< to_format [hex => $key->to_serialized()] >> instead.
-
-=head2 from_wif
+=head3 from_wif
 
 	$key_object = $class->from_wif($str, $network = undef)
 
@@ -220,39 +228,27 @@ key, as this data is included in WIF format.
 
 Returns class instance.
 
-=head2 to_wif
+=head3 to_wif
 
 	$wif_string = $object->to_wif()
 
 Does the opposite of from_wif on a target object
 
-=head2 set_compressed
-
-	$key_object = $object->set_compressed($val)
-
-Change key's compression state to C<$val> (1/0). This will change the WIF
-generated by to_wif() method and also enable creation of uncompressed public
-keys. If C<$val> is omitted it is set to C<1>.
-
-Returns current key instance.
-
-=head2 set_network
-
-	$key_object = $object->set_network($val)
-
-Change key's network state to C<$val>. It can be either network name present in
-Bitcoin::Crypto::Network package or an instance of this class.
-
-Returns current key instance.
-
-=head2 get_public_key
+=head3 get_public_key
 
 	$public_key_object = $object->get_public_key()
 
 Returns instance of L<Bitcoin::Crypto::Key::Public> generated from the private
 key.
 
-=head2 sign_message
+=head3 get_taproot_output_key
+
+	$prv = $object->get_taproot_output_key($tweak_suffix = undef)
+
+Returns a new private key instance that represents an output taproot key.
+Optional C<$tweak_suffix> can be passed as bytestring.
+
+=head3 sign_message
 
 	$signature = $object->sign_message($message)
 
@@ -268,7 +264,7 @@ function to fail. You can encode like this (for UTF-8):
 	use Encode qw(encode);
 	$message = encode('UTF-8', $message);
 
-=head2 sign_transaction
+=head3 sign_transaction
 
 	$object->sign_transaction($tx, %params)
 
@@ -276,6 +272,9 @@ Signs the transaction C<$tx> using this private key. This automatic signing
 only works for standard script types, if your script is non-standard then you
 will have to sign manually.
 
+For taproot, this signs using key path spending. Spending with script path
+needs a custom solution using L</sign_message>.
+
 Note that the module will let you sign any transaction with any private key.
 You have to manually run L<Bitcoin::Crypto::Transaction/verify> to ensure you
 used the right private key and the signature is correct for the corresponding
@@ -321,7 +320,7 @@ is used.
 
 =back
 
-=head2 verify_message
+=head3 verify_message
 
 	$signature_valid = $object->verify_message($message, $signature)
 
