@@ -1,21 +1,25 @@
 package Bitcoin::Crypto::Key::Public;
-$Bitcoin::Crypto::Key::Public::VERSION = '3.000';
+$Bitcoin::Crypto::Key::Public::VERSION = '3.999_1'; # TRIAL
+$Bitcoin::Crypto::Key::Public::VERSION = '3.9991';
 use v5.10;
 use strict;
 use warnings;
 use Moo;
+use Mooish::AttributeBuilder -standard;
 use Types::Common -sigs, -types;
 use Carp qw(carp);
 
 use Bitcoin::Crypto::Script;
 use Bitcoin::Crypto::Base58 qw(encode_base58check);
 use Bitcoin::Crypto::Bech32 qw(encode_segwit);
+use Bitcoin::Crypto::Types -types;
 use Bitcoin::Crypto::Constants;
 use Bitcoin::Crypto::Util qw(hash160 get_public_key_compressed);
+use Bitcoin::Crypto::Helpers qw(ecc);
 
 use namespace::clean;
 
-with qw(Bitcoin::Crypto::Role::BasicKey);
+extends qw(Bitcoin::Crypto::Key::Base);
 
 sub _is_private { 0 }
 
@@ -31,37 +35,64 @@ sub get_hash
 	return hash160($self->to_serialized);
 }
 
-sub key_hash
-{
-	my $self = shift;
-	my $class = ref $self;
-
-	carp "$class->key_hash() is now deprecated. Use $class->get_hash() instead";
-	return $self->get_hash(@_);
-}
-
-around from_serialized => sub {
-	my ($orig, $class, $key) = @_;
-
-	my $self = $class->$orig($key);
-	$self->set_compressed(get_public_key_compressed($key));
-
-	return $self;
-};
-
-signature_for witness_program => (
+signature_for get_xonly_key => (
 	method => Object,
 	positional => [],
 );
 
-sub witness_program
+sub get_xonly_key
 {
 	my ($self) = @_;
 
+	return $self->raw_key('public_xonly');
+}
+
+signature_for from_serialized => (
+	method => Str,
+	positional => [ByteStr],
+);
+
+sub from_serialized
+{
+	my ($class, $key) = @_;
+
+	my $self = $class->SUPER::from_serialized($key);
+	$self->set_compressed(get_public_key_compressed($key));
+
+	return $self;
+}
+
+signature_for witness_program => (
+	method => Object,
+	positional => [PositiveOrZeroInt, {default => 0}, HashRef, {default => sub { {} }}],
+);
+
+sub witness_program
+{
+	state $data_sources = {
+		(Bitcoin::Crypto::Constants::segwit_witness_version) => sub {
+			return shift->get_hash;
+		},
+		(Bitcoin::Crypto::Constants::taproot_witness_version) => sub {
+			my ($self, $params) = @_;
+
+			$self = $self->get_taproot_output_key($params->{tweak_suffix})
+				unless $self->taproot_output;
+
+			return $self->get_xonly_key;
+		},
+	};
+
+	my ($self, $version, $source_data) = @_;
+
+	Bitcoin::Crypto::Exception::SegwitProgram->raise(
+		"can't get witness program data for version $version"
+	) unless exists $data_sources->{$version};
+
 	my $program = Bitcoin::Crypto::Script->new(network => $self->network);
 	$program
-		->add_operation('OP_' . Bitcoin::Crypto::Constants::segwit_witness_version)
-		->push_bytes($self->get_hash);
+		->add_operation("OP_$version")
+		->push_bytes($data_sources->{$version}->($self, $source_data));
 
 	return $program;
 }
@@ -122,9 +153,39 @@ sub get_segwit_address
 		'segwit addresses can only be created with BIP44 in segwit (BIP84) mode'
 	) unless $self->has_purpose(Bitcoin::Crypto::Constants::bip44_segwit_purpose);
 
+	Bitcoin::Crypto::Exception::AddressGenerate->raise(
+		'segwit addresses must not be generated with uncompressed keys to avoid potential fund loss'
+	) unless $self->compressed;
+
 	return encode_segwit($self->network->segwit_hrp, $self->witness_program->run->stack_serialized);
 }
 
+signature_for get_taproot_address => (
+	method => Object,
+	positional => [Maybe [BitcoinScriptTree], {default => undef}],
+);
+
+sub get_taproot_address
+{
+	my ($self, $script_tree) = @_;
+
+	# network field is not required, lazy check for completeness
+	Bitcoin::Crypto::Exception::NetworkConfig->raise(
+		'this network does not support segregated witness'
+	) unless $self->network->supports_segwit;
+
+	Bitcoin::Crypto::Exception::AddressGenerate->raise(
+		'taproot addresses can only be created with BIP44 in taproot (BIP86) mode'
+	) unless $self->has_purpose(Bitcoin::Crypto::Constants::bip44_taproot_purpose);
+
+	my $taproot_program = $self->witness_program(
+		Bitcoin::Crypto::Constants::taproot_witness_version,
+		defined $script_tree ? {tweak_suffix => $script_tree->get_merkle_root} : {}
+	);
+
+	return encode_segwit($self->network->segwit_hrp, $taproot_program->run->stack_serialized);
+}
+
 signature_for get_address => (
 	method => Object,
 	positional => [],
@@ -134,6 +195,9 @@ sub get_address
 {
 	my ($self) = @_;
 
+	return $self->get_taproot_address
+		if $self->has_purpose(Bitcoin::Crypto::Constants::bip44_taproot_purpose);
+
 	return $self->get_segwit_address
 		if $self->has_purpose(Bitcoin::Crypto::Constants::bip44_segwit_purpose);
 
@@ -143,7 +207,7 @@ sub get_address
 	return $self->get_legacy_address
 		if $self->has_purpose(Bitcoin::Crypto::Constants::bip44_purpose);
 
-	return $self->get_segwit_address
+	return $self->get_taproot_address
 		if $self->network->supports_segwit;
 
 	return $self->get_legacy_address;
@@ -176,24 +240,51 @@ Bitcoin::Crypto::Key::Public - Bitcoin public keys
 
 This class allows you to create a public key instance.
 
-You can use a public key to:
+You can use a public key to verify messages and get addresses.
 
-=over
+=head1 INTERFACE
 
-=item * verify messages
+=head2 Attributes
 
-=item * create addresses: legacy (p2pkh), compatibility (p2sh(p2wpkh)) and segwit (p2wpkh).
+=head3 compressed
 
-=back
+Boolean value indicating if this ECC key should be compressed. Default: C<true>.
 
-=head1 METHODS
+I<writer:> C<set_compressed>
 
-=head2 new
+=head3 network
+
+Instance of L<Bitcoin::Crypto::Network> - current network for this key. Can be
+coerced from network id. Default: current default network.
+
+I<writer:> C<set_network>
+
+=head3 purpose
+
+BIP44 purpose which was used to obtain this key. Filled automatically when
+deriving an extended key. If the key was not obtained through BIP44 derivation,
+this attribute is C<undef>.
+
+I<writer:> C<set_purpose>
+
+I<clearer:> C<clear_purpose>
+
+=head3 taproot_output
+
+Boolean value indicating if this key was obtained through taproot tweaking.
+Taproot output keys are used to sign and verify schnorr signatures in P2TR
+outputs. Default: C<false>
+
+I<writer:> C<set_taproot_output>
+
+=head2 Methods
+
+=head3 new
 
 Constructor is reserved for internal and advanced use only. Use L</from_serialized>
 instead.
 
-=head2 from_serialized
+=head3 from_serialized
 
 	$key_object = $class->from_serialized($serialized)
 
@@ -202,60 +293,55 @@ formatable bytestring which must represent a public key in ASN X9.62 format.
 
 Returns a new key object instance.
 
-=head2 to_serialized
+=head3 to_serialized
 
 	$serialized = $key_object->to_serialized()
 
 This returns a public key in ASN X9.62 format. The result is a bytestring which
 can be further formated with C<to_format> utility.
 
-The result will vary depending on compression state: see L</set_compressed>
+The result will vary depending on compression state: see L</compressed>
 
-=head2 from_bytes
-
-Deprecated. Use C<< $class->from_serialized($data) >> instead.
-
-=head2 to_bytes
-
-Deprecated. Use C<< $key->to_serialized() >> instead.
-
-=head2 from_hex
-
-Deprecated. Use C<< $class->from_serialized([hex => $data]) >> instead.
-
-=head2 to_hex
-
-Deprecated. Use C<< to_format [hex => $key->to_serialized()] >> instead.
-
-=head2 get_hash
+=head3 get_hash
 
 	$bytestr = $object->get_hash()
 
 Returns hash160 of the serialized public key.
 
-=head2 key_hash
+=head3 witness_program
 
-Deprecated. Use C<< $key->get_hash() >> instead.
+	$script = $object->witness_program($version, \%args = {})
 
-=head2 set_compressed
+Builds a witness program for given witness C<$version> as
+L<Bitcoin::Crypto::Script> instance. C<%args> depends on witness version:
 
-	$key_object = $object->set_compressed($val)
+=over
 
-Change key's compression state to C<$val> (boolean). This will change the
-address. If C<$val> is omitted it is set to C<1>.
+=item
 
-Returns current key instance.
+For witness version 0 (segwit), no arguments are used.
 
-=head2 set_network
+=item
 
-	$key_object = $object->set_network($val)
+For witness version 1 (taproot), C<tweak_suffix> optional bytestring argument
+can be passed.
 
-Change key's network state to C<$val>. It can be either network name present in
-L<Bitcoin::Crypto::Network> package or an instance of this class.
+=back
 
-Returns current key instance.
+=head3 get_taproot_output_key
 
-=head2 verify_message
+	$pub = $object->get_taproot_output_key($tweak_suffix = undef)
+
+Returns a new public key instance that represents an output taproot key.
+Optional C<$tweak_suffix> can be passed as bytestring.
+
+=head3 get_xonly_key
+
+	$bytestring = $object->get_xonly_key()
+
+Returns a 32-byte bytestring containing the xonly key for this public key.
+
+=head3 verify_message
 
 	$signature_valid = $object->verify_message($message, $signature)
 
@@ -271,7 +357,7 @@ function to fail. You can encode like this (for UTF-8):
 	use Encode qw(encode);
 	$message = encode('UTF-8', $message);
 
-=head2 get_legacy_address
+=head3 get_legacy_address
 
 	$address_string = $object->get_legacy_address()
 
@@ -279,9 +365,9 @@ Returns string containing Base58Check encoded public key hash (C<p2pkh> address)
 
 If the public key was obtained through BIP44 derivation scheme, this method
 will check whether the purpose was C<44> and raise an exception otherwise. If
-you wish to generate this address anyway, call L</clear_purpose>.
+you wish to generate this address anyway, call C<clear_purpose>.
 
-=head2 get_compat_address
+=head3 get_compat_address
 
 	$address_string = $object->get_compat_address()
 
@@ -290,19 +376,36 @@ program for compatibility purposes (C<p2sh(p2wpkh)> address)
 
 If the public key was obtained through BIP44 derivation scheme, this method
 will check whether the purpose was C<49> and raise an exception otherwise. If
-you wish to generate this address anyway, call L</clear_purpose>.
+you wish to generate this address anyway, call C<clear_purpose>.
 
-=head2 get_segwit_address
+=head3 get_segwit_address
 
 	$address_string = $object->get_segwit_address()
 
-Returns string containing Bech32 encoded witness program (C<p2wpkh> address)
+Returns a string containing Bech32 encoded witness version 0 program (C<p2wpkh>
+address)
 
 If the public key was obtained through BIP44 derivation scheme, this method
 will check whether the purpose was C<84> and raise an exception otherwise. If
-you wish to generate this address anyway, call L</clear_purpose>.
+you wish to generate this address anyway, call C<clear_purpose>.
 
-=head2 get_address
+=head3 get_taproot_address
+
+	$address_string = $object->get_taproot_address($script_tree = undef)
+
+Returns a string containing Bech32m encoded witness version 1 program (C<p2tr>
+address)
+
+Optional C<$script_tree> can be passed as L<Bitcoin::Crypto::Script::Tree>
+object. Passing this argument will generate an address that can be spent using
+script path spend as well as key path spend. If this argument is not passed, an
+unspendable script path will be used accord to BIP341.
+
+If the public key was obtained through BIP44 derivation scheme, this method
+will check whether the purpose was C<86> and raise an exception otherwise. If
+you wish to generate this address anyway, call C<clear_purpose>.
+
+=head3 get_address
 
 	$address_string = $object->get_address()
 
@@ -315,23 +418,17 @@ most fitting:
 matches the purpose
 
 =item * If the key doesn't have a purpose but the network supports segwit,
-returns a segwit address (same as C<get_segwit_address>)
+returns a taproot address (same as C<get_taproot_address>, but does not accept
+a script tree)
 
 =item * If the network doesn't support segwit, returns legacy address
 
 =back
 
 B<NOTE>: The rules this function uses to choose the address type B<will>
-change when more up-to-date address types are implemented (like taproot). Use
+change when more up-to-date address types are implemented. Use
 other address functions if this is not what you want.
 
-=head2 clear_purpose
-
-	$object->clear_purpose;
-
-Clears the BIP44 purpose of this key instance, removing safety checks on
-address generation.
-
 =head1 EXCEPTIONS
 
 This module throws an instance of L<Bitcoin::Crypto::Exception> if it
@@ -354,6 +451,8 @@ L<Bitcoin::Crypto::Exception> namespace:
 
 L<Bitcoin::Crypto::Key::Private>
 
+L<Bitcoin::Crypto::Key::NUMS>
+
 L<Bitcoin::Crypto::Base58>
 
 L<Bitcoin::Crypto::Bech32>
