@@ -1,5 +1,6 @@
 package Bitcoin::Crypto::Network;
-$Bitcoin::Crypto::Network::VERSION = '3.000';
+$Bitcoin::Crypto::Network::VERSION = '3.999_1'; # TRIAL
+$Bitcoin::Crypto::Network::VERSION = '3.9991';
 use v5.10;
 use strict;
 use warnings;
@@ -9,6 +10,7 @@ use Mooish::AttributeBuilder -standard;
 use Types::Common -sigs, -types;
 
 use Bitcoin::Crypto::Exception;
+use Bitcoin::Crypto::Types -types;
 
 use namespace::clean;
 
@@ -25,15 +27,15 @@ has param 'name' => (
 );
 
 has param 'p2pkh_byte' => (
-	isa => StrLength [1, 1],
+	coerce => ByteStrLen [1],
 );
 
 has param 'wif_byte' => (
-	isa => StrLength [1, 1],
+	coerce => ByteStrLen [1],
 );
 
 has param 'p2sh_byte' => (
-	isa => StrLength [1, 1],
+	coerce => ByteStrLen [1],
 	required => 0,
 );
 
