@@ -1,5 +1,6 @@
 package Bitcoin::Crypto::PSBT::Field;
-$Bitcoin::Crypto::PSBT::Field::VERSION = '3.000';
+$Bitcoin::Crypto::PSBT::Field::VERSION = '3.999_1'; # TRIAL
+$Bitcoin::Crypto::PSBT::Field::VERSION = '3.9991';
 use v5.10;
 use strict;
 use warnings;
@@ -49,11 +50,11 @@ sub BUILD
 		$self->_set_raw_key(undef);
 	}
 
-	if (exists $args->{value}) {
+	if (defined $args->{value}) {
 		$self->set_value($args->{value});
 	}
 
-	if (exists $args->{key}) {
+	if (defined $args->{key}) {
 		$self->set_key($args->{key});
 	}
 
