@@ -1,5 +1,6 @@
 package Bitcoin::Crypto::Role::Key;
-$Bitcoin::Crypto::Role::Key::VERSION = '3.000';
+$Bitcoin::Crypto::Role::Key::VERSION = '3.999_1'; # TRIAL
+$Bitcoin::Crypto::Role::Key::VERSION = '3.9991';
 use v5.10;
 use strict;
 use warnings;
@@ -31,7 +32,7 @@ requires qw(
 	_is_private
 );
 
-sub BUILD
+sub _validate_key
 {
 	my ($self) = @_;
 	my $entropy = $self->key_instance;
@@ -51,6 +52,17 @@ sub BUILD
 			'private key is not valid'
 		) unless ecc->verify_private_key(ensure_length $entropy, Bitcoin::Crypto::Constants::key_max_length);
 	}
+	else {
+		Bitcoin::Crypto::Exception::KeyCreate->raise(
+			'public key is not valid'
+		) unless ecc->verify_public_key($entropy);
+	}
+}
+
+sub BUILD
+{
+	my ($self) = @_;
+	$self->_validate_key;
 }
 
 signature_for has_purpose => (
@@ -67,36 +79,58 @@ sub has_purpose
 
 signature_for raw_key => (
 	method => Object,
-	positional => [Maybe [Enum [qw(private public public_compressed)]], {default => undef}],
+	positional => [Maybe [Enum [qw(private public public_compressed public_xonly)]], {default => undef}],
 );
 
+# helpers for raw_key
+sub __full_private
+{
+	my ($self, $key) = @_;
+	return ensure_length $key, Bitcoin::Crypto::Constants::key_max_length;
+}
+
+sub __private_to_public
+{
+	my ($self, $key) = @_;
+	return ecc->create_public_key($self->__full_private($key));
+}
+
+sub __public_compressed
+{
+	my ($self, $key, $compressed) = @_;
+	return ecc->compress_public_key($key, $compressed);
+}
+
 sub raw_key
 {
 	my ($self, $type) = @_;
 	my $is_private = $self->_is_private;
+	my $key = $self->key_instance;
 
-	$type = $is_private ? 'private' : 'public'
-		unless defined $type;
+	$type //= $is_private ? 'private' : 'public';
+	if ($type eq 'public' && (!$self->does('Bitcoin::Crypto::Role::Compressed') || $self->compressed)) {
+		$type = 'public_compressed';
+	}
 
 	if ($type eq 'private') {
 		Bitcoin::Crypto::Exception::KeyCreate->raise(
 			'cannot create private key from a public key'
 		) unless $is_private;
 
-		return ensure_length $self->key_instance, Bitcoin::Crypto::Constants::key_max_length;
+		return $self->__full_private($key);
 	}
-	else {
-		my $compressed = $self->does('Bitcoin::Crypto::Role::Compressed') ? $self->compressed : !!1;
-		$compressed = !!1 if $type eq 'public_compressed';
-
-		my $key = $self->key_instance;
-		$key = ecc->create_public_key($key)
+	elsif ($type eq 'public_xonly') {
+		$key = $self->__private_to_public($key)
 			if $is_private;
 
-		return ecc->compress_public_key($key, $compressed);
+		return ecc->xonly_public_key($self->__public_compressed($key, 1));
 	}
+	else {
+		$key = $self->__private_to_public($key)
+			if $is_private;
 
-	# no need to check for invalid input, since we have a signature with enum
+		return $self->__public_compressed($key, $type eq 'public_compressed');
+	}
 }
 
 1;
