@@ -1,5 +1,6 @@
 package Bitcoin::Crypto::Script::Recognition;
-$Bitcoin::Crypto::Script::Recognition::VERSION = '3.000';
+$Bitcoin::Crypto::Script::Recognition::VERSION = '3.999_1'; # TRIAL
+$Bitcoin::Crypto::Script::Recognition::VERSION = '3.9991';
 use v5.10;
 use strict;
 use warnings;
@@ -12,6 +13,8 @@ use Try::Tiny;
 
 use Bitcoin::Crypto::Script::Opcode;
 
+use namespace::clean;
+
 has param 'script' => (
 	isa => InstanceOf ['Bitcoin::Crypto::Script'],
 );
